"""This module contains `docker events` class"""

import pystache
import arrow

from .command import Command


class Events(Command):
    """This class implements `docker events` command"""

    name = "events"
    require = []

    defaultTemplate = '{{Time}}\t{{Type}}\t{{Action}}\t{{{Actor}}}'

    def __init__(self):
        Command.__init__(self)
        self.settings[self.name] = None

    def eval_command(self, args):
        if args["format"] is None:
            fm = self.defaultTemplate
        else:
            fm = args["format"]
        del args["format"]

        args["filters"] = dict(args["filters"]) if args["filters"] else None

        args["decode"] = True
        for event in self.client.events(**args):
            event["Time"] = arrow.get(event["time"])
            print pystache.render(fm, event)
        self.settings[self.name] = ""

    def final(self):
        return self.settings[self.name]



"""This module contains `docker container logs` class"""

from .command import Command


class Logs(Command):
    """This class implements `docker container logs` command"""

    name = "container logs"
    require = []

    def __init__(self):
        Command.__init__(self)
        self.settings[self.name] = None

    def eval_command(self, args):
        if args["since"] == 0:
            # since cannot be 0, ref: https://github.com/docker/docker-py/blob/9d23278643cb6b4a097e833915a73ab9a2eba10d/docker/api/container.py#L837
            args["since"] = None
        if args["tail"] != 'all':
            args["tail"] = int(args["tail"])
        if args["follow"]:
            args["stream"] = True
            for line in self.client.logs(**args):
                print line,
            self.settings[self.name] = ""
        else:
            logs = self.client.logs(**args)
            self.settings[self.name] = logs if logs else "\r"

    def final(self):
        return self.settings[self.name]



"""Addon Loader module"""

import os
import pkgutil
import imp
from ast import literal_eval
from ConfigParser import RawConfigParser
from git import RemoteProgress

from ..Utils.deepgetattr import deepgetattr
from ..Utils.hightlight import hightlight_python
from ..Utils.urlutil import is_git_url


class Progress(RemoteProgress):
    """Git Progree Handler"""

    def line_dropped(self, line):
        print line

    def update(self, op_code, cur_count, max_count=None, message=''):
        print "{}/{}".format(cur_count, max_count)


class Loader(object):
    """Addon Loader"""
    inspects = {}

    def __init__(self, argparser=None, tsaotun_global_config="{}/Tsaotun/config.ini".format(os.path.expanduser("~")), addon_path="{}/Tsaotun/addons/".format(os.path.expanduser("~"))):
        self.argparser = argparser
        self.__locate(addon_path)
        try:
            if not os.path.exists(tsaotun_global_config):
                open(tsaotun_global_config, 'a').close()
            self.tsaotun_global_config = tsaotun_global_config
        except OSError as e:
            raise RuntimeError(
                "{}".format(str(e)))

    def __locate(self, addon_path):
        """Locate addon path"""
        try:
            if not os.path.exists(addon_path):
                os.makedirs(addon_path)
            self.addon_path = addon_path
        except OSError as e:
            raise RuntimeError(
                "{}".format(str(e)))

    def __parse_args(self, configs):
        """Parsing __argparse__ and assign to argparser"""
        if self.argparser:
            for config in configs:
                if config["position"] == 'Self':
                    parser = self.argparser[
                        config["namespace"]][config["position"]]
                    for action in config["actions"]:
                        exec("import textwrap; parser.{}".format(action), {'parser': parser})
                    self.argparser[config["namespace"]][
                        config["position"]] = parser
                else:
                    if config["subcommand"]:
                        parser = self.argparser[config["namespace"]][
                            config["position"]].choices.get(config["subcommand"])
                        for action in config["actions"]:
                            exec("import textwrap; parser.{}".format(action), {'parser': parser})
                        self.argparser[config["namespace"]][
                            config["position"]].choices.update({config["subcommand"]: parser})
                    else:
                        parser = self.argparser[
                            config["namespace"]][config["position"]]
                        subparser = None
                        for i, action in enumerate(config["actions"]):
                            if i == 0:
                                exec(
                                    "import textwrap; import argparse; global subparser; subparser = parser.{}".format(action))
                            else:
                                exec("import textwrap; global subparser; subparser.{}".format(action))
                        self.argparser[config["namespace"]][
                            config["position"]] = parser
        else:
            raise RuntimeError("argparser is not specified.")

    def __parse_config(self, section, options=None):
        """Parsing global config file"""
        cfg = RawConfigParser()
        with open(self.tsaotun_global_config) as f:
            cfg.readfp(f, self.tsaotun_global_config)
        if cfg.has_section(section) is False:
            cfg.add_section(section)
        if options:
            stored = [cfg.get(section, option) if cfg.has_option(section, option) else cfg.set(
                section, option, 'False') is True for option in options]
        else:
            stored = cfg.items(section)
        with open(self.tsaotun_global_config, 'wb') as f:
            cfg.write(f)
        return stored

    def __active(self, addon_names):
        """Check if the given addon is active"""
        return self.__parse_config('addon', addon_names)

    def load(self):
        """Load addons"""
        addon_names = [n for _, n, _ in pkgutil.iter_modules(
            ["{}".format(self.addon_path)])]
        addons = {}
        for addon_name, active in zip(addon_names, self.__active(addon_names)):
            try:
                f, filename, description = imp.find_module(
                    addon_name, [self.addon_path])
                mod = imp.load_module(
                    addon_name, f, filename, description)
                self.inspects[addon_name] = ''.join(
                    [mod.__file__.split('.')[0], '.py'])
                if literal_eval(active):
                    for k, v in mod.__override__.iteritems():
                        addons["{}|{}".format(k, v)] = deepgetattr(
                            mod, "{}.{}".format(k, v))
                    self.__parse_args(mod.__argparse__)
            except ImportError as e:
                raise RuntimeError(str(e))

        return addons, self.argparser

    def enable(self, addon):
        """Enable an addon"""
        try:
            if self.inspects[addon]:
                cfg = RawConfigParser()
                with open(self.tsaotun_global_config) as f:
                    cfg.readfp(f, self.tsaotun_global_config)
                cfg.set('addon', addon, 'True')
                with open(self.tsaotun_global_config, 'wb') as f:
                    cfg.write(f)
                return "Done!"
        except KeyError as e:
            raise RuntimeError("No such addon: {}".format(str(e)))

    def disable(self, addon):
        """Disable an addon"""
        try:
            if self.inspects[addon]:
                cfg = RawConfigParser()
                with open(self.tsaotun_global_config) as f:
                    cfg.readfp(f, self.tsaotun_global_config)
                cfg.set('addon', addon, 'False')
                with open(self.tsaotun_global_config, 'wb') as f:
                    cfg.write(f)
                return "Done!"
        except KeyError as e:
            raise RuntimeError("No such addon: {}".format(str(e)))

    def inspect(self, addon):
        """Inspect an addon"""
        try:
            args = {'sourcefile': self.inspects[addon]}
            return hightlight_python(args)
        except KeyError as e:
            raise RuntimeError("No such addon: {}".format(str(e)))

    def install(self, url, addon):
        """Install an addon from a given url"""
        if is_git_url(url):
            from git import (Repo, GitCommandError)
            if addon is None:
                addon = url.split('/')[-1].split('.')[0]
            dest = os.path.join(self.addon_path, addon)
            try:
                Repo.clone_from(url, dest, progress=Progress())
                if 'requirements.txt' in os.listdir(dest):
                    os.system("pip install -r {}".format(os.path.join(dest, 'requirements.txt')))
                return "'{}' successfully installed!".format(addon)
            except GitCommandError as e:
                raise RuntimeError("{}".format(str(e)))
            except IOError:
                print "Please run 'pip install -r {}' to install the needed requirements, and then the installation of {} is done.".format(os.path.join(dest, 'requirements.txt'), addon)
        else:
            raise RuntimeError("Invalid URL: {}".format(url))

    def rm(self, addon):
        """Remove an addon"""
        import shutil
        try:
            shutil.rmtree(os.path.join(self.addon_path, addon))
            return "'{}' successfully removed!".format(addon)
        except OSError as e:
            raise RuntimeError("{}".format(str(e)))

    def addons(self):
        """List addons"""
        addon_names = [n for _, n, _ in pkgutil.iter_modules(
            ["{}".format(self.addon_path)])]
        return [{'Addon': n, 'Enabled': a} for n, a in zip(addon_names, self.__active(addon_names))]



"""python setup script"""
import codecs
import os
import re
import shutil
import platform

from setuptools import (setup, find_packages)
from setuptools.command.install import install

ROOT_DIR = os.path.dirname(__file__)


class PostInstallCommand(install):
    """Post-installation for installation mode"""

    def run(self):
        install.run(self)
        print "Installing auto completion of tsaotun to",
        src = os.path.join(ROOT_DIR, 'completion', 'tsaotun')
        sys = platform.system()
        try:
            if sys == 'Darwin':
                dest = os.path.join(
                    os.popen('brew --prefix').read()[:-1], 'etc', 'bash_completion.d', 'tsaotun')
                print dest
                shutil.copy(src, dest)
            elif sys == 'Linux':
                dest = os.path.join(
                    '/etc', 'bash_completion.d', 'tsaotun')
                print dest
                shutil.copy(src, dest)
            else:  # Windows, etc.
                print "... \n Warning: {} is currently not supported. Skipped.".format(sys)
        except IOError:
            print "Permission denied: You probably want to copy '{}' to '{}' manually.".format(src, dest)
        print "Tsaotun is installed successfully."


def find_version(*file_paths):
    """
    Read the version number from a source file.
    Why read it, and not import?
    see https://groups.google.com/d/topic/pypa-dev/0PkjVpcxTzQ/discussion
    """
    # Open in Latin-1 so that we avoid encoding errors.
    # Use codecs.open for Python 2 compatibility
    with codecs.open(os.path.join(ROOT_DIR, *file_paths), 'r', 'utf-8') as f:
        version_file = f.read()

    # The version line must have the form
    # __version__ = 'ver'
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string')


def read_requirements(requirements):
    """Read the requirements from a requirements txt file"""
    try:
        with codecs.open(os.path.join(ROOT_DIR, requirements), 'r', 'utf-8') as f:
            return f.read().splitlines()
    except IOError:
        raise IOError(os.getcwd())

# Get the long description from the relevant file
with codecs.open(os.path.join(ROOT_DIR, 'README.rst'), 'r', 'utf-8') as f:
    long_description = f.read()

setup(name='tsaotun',
      version=find_version('tsaotun', '__init__.py'),
      description='Python based Assistance for Docker',
      long_description=long_description,
      author='Boik Su',
      author_email='boik@tdohacker.org',
      url='https://github.com/qazbnm456/tsaotun',
      download_url='https://codeload.github.com/qazbnm456/tsaotun/tar.gz/0.9.4',
      keywords=['0.9.4'],
      packages=find_packages(),
      install_requires=read_requirements('requirements.txt'),
      entry_points="""
            [console_scripts]
            tsaotun=tsaotun.cli:cli
      """,
      classifiers=[
          'Environment :: Console',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: Apache Software License',
          'Topic :: Software Development',
          'Topic :: Terminals',
          'Topic :: Utilities',
          'Operating System :: OS Independent',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7'
      ],
      cmdclass={
          'install': PostInstallCommand,
      }
      )
